/* esSerpenteNumeri_v2.cpp

   Scrivere una funzione int** crea_serpente(int r, int c). 
   La funzione crea una matrice di dimensione r*c fatta nel 
   seguente modo (es. r=5, c=6):
   1  2  3  4  5  6
   12 11 10 9  8  7
   13 14 15 16 17 18
   24 23 22 21 20 19
   25 26 27 28 29 30
   La matrice deve essere restituita al chiamante come 
   valore di ritorno.

   Scrivere un programma che:
   - chiede all'utente di inserire da tastiera due interi N e M; 
   - invoca la funzione passandole N e M come argomenti;
   - stampa a video il contenuto della matrice restituita dalla funzione. 

   Nota: Per il corretto spaziamento dei numeri della matrice, 
   usare il carattere tabulazione '\t'.

   Output di esempio:
   Inserisci N: 5
   Inserisci M: 6

   1       2       3       4       5       6
   12      11      10      9       8       7
   13      14      15      16      17      18
   24      23      22      21      20      19
   25      26      27      28      29      30

*********************************************************/

#include <iostream>
using namespace std;

int** crea_serpente(int r, int c) {
    // dichiaro la matrice come doppio puntatore, ovvero
    // come un puntatore di puntatore a interi
    int** M;
    
    // alloco memoria per l'array di primo livello (righe della matrice)
    // r elementi di tipo int*
    M = new int*[r];
    for (int i=0; i<r; i++) {
        // per ogni elemento dell'array, devo allocare memoria (colonne della matrice)
        // c elementi di tipo int
        M[i] = new int[c];
    }
    
    int cont = 1;
    for (int i=0; i<r; i++) {
        // per le righe pari, stampo da sinistra a destra
        // per le righe dispari, stampo da destra a sinistra
        if (i%2 == 0) {
            for (int j=0; j<c; j++)
                M[i][j] = cont++;  // incremento postfisso
        }
        else {
            for (int j=c-1; j>=0; j--)
                M[i][j] = cont++;  // incremento postfisso
        }
    }
    
    return M;
}

int main() {
    int N, M;
    cout << "Inserisci N: ";
    cin >> N;
    cout << "Inserisci M: ";
    cin >> M;
    
    // invoco la funzione, assegnando il risultato
    // a un doppio puntatore
    int** mat = crea_serpente(N,M);
    
    // stampo la matrice
    cout << endl;
    for (int i=0; i<N; i++) {
        for (int j=0; j<M; j++) {
            // allineo la stampa del numero su 4 caratteri
            cout << mat[i][j] << "\t";
        }
        cout << endl;
    }
    
    // deallocazione della memoria dinamica
    for (int i=0; i<N; i++)
        delete[] mat[i];
    delete[] mat;
    
    return 0;
}

/* Alternativa: funzione separata per la stampa della matrice:
   NOTA: Il vettore di vettori non puo' essere passato come "const int** M",
         perche' questo tipo non garantirebbe che gli elementi interi siano
         veramente costanti. Infatti, la funzione stampa_matrice() potrebbe
         modificare il puntatore M[0] a puntare ad un'altra riga, magari allocata
         dalla funzione stessa. In questo modo gli elementi M[0][0], M[0][1],
         etc. risulterebbero diversi rispetto a prima dell'invocazione.
         Il tipo "const int* const* M" e' invece permesso, perche' impone
         che anche M[0] (ed M[1], M[2], etc., in pratica tutto il "direttorio
         delle righe") sia costante all'interno della funzione.
void stampa_matrice(const int* const* M, int r, int c){
    for (int i=0; i<r; i++) {
        for (int j=0; j<c; j++) {
            // allineo la stampa del numero su 4 caratteri
            cout << mat[i][j] << "\t";
        }
        cout << endl;
    }
}
   Nel main:
   ...
   stampa_matrice(mat, N, M);
   ...
*/